import React from 'react'
import echarts from 'echarts'
import {
  getLegendOption,
  getSeriesOption,
} from './generator'

interface IProps {
  options: Ioptions
}

const generator = (option: Ioptions) => {
  const { chartStyles, data, cols } = option
  const { legend, label, spec } = chartStyles

  return {
    tooltip : {
      trigger: 'item',
      formatter: '{b}<br/>{c} ({d}%)',
    },
    legend: getLegendOption('pie', legend, data, cols),
    series: getSeriesOption('pie', data, label, cols, spec),
  }
}

let chart: any
const initChart = (type: string, id: HTMLDivElement, option: Ioptions) => {
  if (type === 'init') {
    chart = echarts.init(id)
  }
  chart.setOption(generator(option))
}

class Chart extends React.Component<IProps, any> {
  div: HTMLDivElement

  componentDidMount() {
    const { options } = this.props
    initChart('init', this.div, options)
  }

  componentDidUpdate() {
    const { options } = this.props
    initChart('update', this.div, options)
  }

  render() {
    const { width, height } = this.props.options
    return (
      <div style={{ width, height }} ref={(node) => this.div = node} />
    )
  }
}

export default Chart
