import React from 'react'
import echarts from 'echarts'
import {
  getTooltipOption,
  getLegendOption,
  getXAxisOption,
  getYAxisOption,
  getSeriesOption,
} from './generator'

interface IProps {
  options: Ioptions
}

const generator = (option: Ioptions) => {
  const { chartStyles, data, cols } = option
  const { xAxis, yAxis, legend, splitLine, label, spec } = chartStyles

  return {
    tooltip: getTooltipOption('bar'),
    legend: getLegendOption('bar', legend, data, cols),
    xAxis: getXAxisOption(xAxis, data, cols, splitLine),
    yAxis: getYAxisOption(yAxis, data, splitLine, cols),
    series: getSeriesOption('bar', data, label, cols, spec),
  }
}

let chart: any
const initChart = (type: string, id: HTMLDivElement, option: Ioptions) => {
  if (type === 'init') {
    chart = echarts.init(id)
  }
  chart.setOption(generator(option))
}

class Chart extends React.Component<IProps, any> {
  div: HTMLDivElement

  componentDidMount() {
    const { options } = this.props
    initChart('init', this.div, options)
  }

  componentDidUpdate() {
    const { options } = this.props
    initChart('update', this.div, options)
  }

  render() {
    const { width, height } = this.props.options
    return (
      <div style={{ width, height }} ref={(node) => this.div = node} />
    )
  }
}

export default Chart
