import { chartType } from '../../util/config'
import { defaultFontSize } from '../../util/defalut.config'

/**
 * get legend | series key
 * @param {Array} data
 * @return {Array}
 */
const dataKey = (data: any[], cols: any[]) => {
  const name = []
  if (data && data.length > 0) {
    const { ...value } = data[0]
    for (const key in value) {
      if (key !== cols[0]) {
        name.push(key.match(/\(.+?\)/g)[0].slice(1, -1))
      }
    }
  }
  return name
}

/**
 * get data array value
 * @param {Array} data
 * @return {Object}
 */
const dataValue = (data: any[], cols: any[]) => {
  if (data && data.length > 0) {
    const { ...value } = data[0]
    const obj: any = {}
    for (const key in value) {
      if (key !== cols[0]) {
        const name = key.match(/\(.+?\)/g)[0].slice(1, -1)
        obj[name] = data.map((item) => item[key])
      }
    }
    return obj
  }
}

/**
 * get cols array value
 * @param {Array} data
 * @param {Array} cols
 * @return {Array}
 */
const colsValue = (data: any[], cols: any[]) => {
  if (data && data.length > 0) {
    return data.map((item) => item[cols[0]])
  }
  return []
}

/**
 * legend select
 * @param {boolean} selectAll
 * @param {Array} data
 * @return {Object}
 */
const legendSelected = (type: string, selectAll: boolean, data: any[], cols: any[]) => {
  const obj: any = {}
  if (chartType.indexOf(type) > -1) {
    dataKey(data, cols).map((item) => {
      obj[item] = selectAll ? true : false
    })
  }

  if (type === 'pie') {
    colsValue(data, cols).map((item) => {
      obj[item] = selectAll ? true : false
    })
  }

  return obj
}

const getTooltipOption = (type: string) => {
  if (type === 'linebar') {
    return {
      show: true,
      trigger: 'axis',
      axisPointer: {
        type: 'cross',
      },
    }
  }
  return {
    show: true,
    trigger: 'axis',
    axisPointer: {
      type: 'shadow',
    },
  }
}

/**
 * legend option
 * @param {Ilegend} legend
 * @param {Array} data
 * @return {Object}
 */
const getLegendOption = (type: string, legend: Ilegend, data: any[], cols: any[]) => {
  const {
    color,
    fontFamily,
    fontSize,
    legendPosition,
    selectAll,
    showLegend,
  } = legend

  let position
  switch (legendPosition) {
    case 'right':
      position = { top: 0, right: '10%' }
      break
    case 'left':
      position = { top: 0, left: '10%' }
      break
    case 'top':
      position = { bottom: 0, left: '10%' }
      break
    case 'bottom':
      position = { bottom: 0, right: '10%' }
      break
    default:
      position = { right: '10%' }
  }

  let dataSelected
  if (type === 'pie') {
    dataSelected = {
      data: colsValue(data, cols),
      selected: legendSelected(type, selectAll, data, cols),
      orient: 'vertical',
    }
  }
  if (chartType.indexOf(type) > -1) {
    dataSelected = {
      data: dataKey(data, cols),
      selected: legendSelected(type, selectAll, data, cols),
    }
  }

  return {
    show: showLegend,
    textStyle: {
      fontFamily,
      fontSize: fontSize || defaultFontSize,
      color,
    },
    ...position,
    ...dataSelected,
  }
}

/**
 * xAxis option
 * @param {Array} data
 * @return {Object}
 */
const getXAxisOption = (xAxis: IxAxis, data: any[], cols: any[], splitLine: IsplitLine) => {
  const {
    showLabel,
    labelColor,
    labelFontFamily,
    labelFontSize,
    showLine,
    lineColor,
    lineSize,
    lineStyle,
    xAxisInterval,
    xAxisRotate,
  } = xAxis

  const {
    showVerticalLine,
    verticalLineColor,
    verticalLineSize,
    verticalLineStyle,
  } = splitLine

  return [{
    data: colsValue(data, cols),
    interval: xAxisInterval,
    offset: xAxisRotate,

    // x轴 线
    axisLine: {
      show: showLine,
      lineStyle: {
        color: lineColor,
        width: +lineSize,
        type: lineStyle,
      },
    },

    // x轴 标签
    axisLabel: {
      show: showLabel,
      color: labelColor,
      fontFamily: labelFontFamily,
      fontSize: +labelFontSize || defaultFontSize,
    },

    // 垂直分割线
    splitLine: {
      show: showVerticalLine,
      lineStyle: {
        color: verticalLineColor,
        width: +verticalLineSize,
        type: verticalLineStyle,
      },
    },
  }]
}

/**
 * yAxis option
 * @param {IyAxis} yAxis
 * @return {Object}
 */
const getYAxisOption = (yAxis: IyAxis, data: any[], splitLine: IsplitLine, cols: any[]) => {
  const {
    showLabel,
    labelColor,
    labelFontFamily,
    labelFontSize,
    showLine,
    lineColor,
    lineSize,
    lineStyle,
    showTitleAndUnit,
    nameGap,
    nameLocation,
    nameRotate,
    titleColor,
    titleFontFamily,
    titleFontSize,
    inverse,
  } = yAxis

  const {
    showHorizontalLine,
    horizontalLineColor,
    horizontalLineSize,
    horizontalLineStyle,
  } = splitLine

  const name = []
  if (data && data.length > 0) {
    const { ...value } = data[0]
    for (const key in value) {
      if (key !== cols[0]) {
        name.push(key.match(/\(.+?\)/g)[0].slice(1, -1))
      }
    }
  }

  return {
    // y轴 标题和单位
    name: showTitleAndUnit && name[0] || '',
    nameGap,
    nameLocation,
    nameRotate,
    inverse,
    nameTextStyle: {
      color: titleColor,
      fontFamily: titleFontFamily,
      fontSize: +titleFontSize || defaultFontSize,
    },

    // y轴 线
    axisLine: {
      show: showLine,
      lineStyle: {
        color: lineColor,
        width: +lineSize,
        type: lineStyle,
      },
    },

    // y轴 标签
    axisLabel: {
      show: showLabel,
      color: labelColor,
      fontFamily: labelFontFamily,
      fontSize: +labelFontSize || defaultFontSize,
    },

    // 水平分割线
    splitLine: {
      show: showHorizontalLine,
      lineStyle: {
        color: horizontalLineColor,
        width: +horizontalLineSize,
        type: horizontalLineStyle,
      },
    },
  }
}

/**
 * series option
 * @param {Array} data
 * @return {Array}
 */
const getSeriesOption =
  (type: string, data: any[], label: Ilabel, cols: any[], spec?: Ispec, mixedlinebar?: Imixedlinebar) => {
  const {
    labelColor,
    labelFontFamily,
    labelFontSize,
    labelPosition,
    showLabel,
    pieLabelPosition,
  } = label

  const {
    roseType,
    circle,
    smooth,
    step,
  } = spec

  if (chartType.indexOf(type) > -1) {
    const smoothOption = spec.hasOwnProperty('smooth') ? { smooth } : null
    const stepOption = spec.hasOwnProperty('step') ? { step } : null

    return dataKey(data, cols).map((item) => {
      return {
        name: item,
        type,
        data: dataValue(data, cols)[item],
        label: {
          show: showLabel,
          color: labelColor,
          fontSize: labelFontSize || defaultFontSize,
          fontFamily: labelFontFamily,
          position: labelPosition,
        },
        ...smoothOption,
        ...stepOption,
      }
    })
  }

  if (type === 'stackbar') {
    return dataKey(data, cols).map((item) => {
      return {
        name: item,
        type: 'bar',
        stack: '总量',
        data: dataValue(data, cols)[item],
        label: {
          show: showLabel,
          color: labelColor,
          fontSize: labelFontSize || defaultFontSize,
          fontFamily: labelFontFamily,
          position: labelPosition,
        },
      }
    })
  }

  if (type === 'linebar') {
    const smoothOption = spec.hasOwnProperty('smooth') ? { smooth } : null
    const stepOption = spec.hasOwnProperty('step') ? { step } : null

    return dataKey(data, cols).map((item, index) => {
      return {
        name: item,
        type: mixedlinebar[item] || 'bar',
        // yAxisIndex: index,
        data: dataValue(data, cols)[item],
        label: {
          show: showLabel,
          color: labelColor,
          fontSize: labelFontSize || defaultFontSize,
          fontFamily: labelFontFamily,
          position: labelPosition,
        },
        ...smoothOption,
        ...stepOption,
      }
    })
  }

  if (type === 'pie') {
    const datakey = dataKey(data, cols)[0] // 一个指标
    const datavalue = dataValue(data, cols)
    const colsvalue = colsValue(data, cols)
    const name = colsvalue.map((item, index) => {
      return {
        name: item,
        value: datavalue[datakey][index],
      }
    })

    return {
      type: 'pie',
      radius : circle ? ['50%', '70%'] : '70%', // 环形图
      data: name,
      roseType, // 南丁格儿图
      itemStyle: {
          emphasis: {
              shadowBlur: 10,
              shadowOffsetX: 0,
              shadowColor: 'rgba(0, 0, 0, 0.5)',
          },
      },
      label: {
        show: showLabel,
        position: pieLabelPosition,
        color: labelColor,
        fontFamily: labelFontFamily,
        fontSize: labelFontSize || defaultFontSize,
      },
    }
  }
}

export {
  getTooltipOption,
  getLegendOption,
  getXAxisOption,
  getYAxisOption,
  getSeriesOption,
}
