import React from 'react'
import { BarChart, Bar, Cell, XAxis, YAxis, CartesianGrid, Tooltip, Legend } from 'recharts'

interface IProps {
  options: Ioptions
}

interface IState {

}

/**
 * @description if (!cols) { return 1, 2, 3... }
 * @param {Array} cols
 * @return {String | None} The xAxis label
 */
const dataKey = (cols: any[]) => {
  if (cols && cols.length > 0) {
    return cols[0]
  }
  return null
}

/**
 * @description yAxis label
 * @param {IyAxis} yAxis
 * @return {Object} label content
 */
const label: any = (yAxis: IyAxis, data: any[]) => {
  if (!yAxis.showTitleAndUnit) {
    return null
  }

  let position = 'left'
  let name = null
  if (yAxis.nameLocation === 'start') {
    position = 'insideTopLeft'
  }

  if (yAxis.nameLocation === 'end') {
    position = 'insideBottomLeft'
  }

  if (data && data.length > 0) {
    const { id, ...value } = data[0]
    for (const key in value) {
      name = key.match(/\(.+?\)/g)[0].slice(1, -1)
    }
  }

  return {
    value: name,
    position,
    offset: yAxis.nameGap,
  }
}

class Chart extends React.Component<IProps, IState> {

  grid() {
    const { chartStyles } = this.props.options
    const { splitLine } = chartStyles
    return (
      <CartesianGrid
        horizontal={splitLine.showHorizontalLine}
        vertical={splitLine.showVerticalLine}
      />
    )
  }

  xAxis() {
    const { chartStyles, cols } = this.props.options
    const { xAxis } = chartStyles
    return (
      <XAxis
        hide={!xAxis.showLabel && !xAxis.showLine}
        dataKey={dataKey(cols)}
        interval={xAxis.xAxisInterval}
        axisLine={xAxis.showLine}
        tickLine={xAxis.showLabel}
      />
    )
  }

  yAxis() {
    const { chartStyles, data } = this.props.options
    const { yAxis } = chartStyles
    return (
      <YAxis
        hide={!yAxis.showLabel && !yAxis.showLine}
        axisLine={yAxis.showLine}
        tickLine={yAxis.showLabel}
        label={label(yAxis, data)}
      />
    )
  }

  legend() {
    const { chartStyles } = this.props.options
    const { legend } = chartStyles

    if (!legend.showLegend) {
      return null
    }

    return (
      <Legend
        layout='horizontal'
        // align={legend.legendPosition}
        verticalAlign='bottom'
      />
    )
  }

  bar() {
    const { data } = this.props.options
    if (data && data.length > 0) {
      const { id, ...value } = data[0]
      const arr = []
      for (const key in value) {
        arr.push(key)
      }
      return arr.map((item, index) => <Bar key={index} dataKey={item} fill='#8884d8' />)
    }
  }

  render() {
    const { width, height, data } = this.props.options
    if (!width || !height) {
      return
    }

    return (
      <BarChart
        width={width}
        height={height}
        data={data}
        margin={{
          top: 5, right: 30, left: 20, bottom: 5,
        }}
      >
        <Tooltip />
        {this.grid()}
        {this.xAxis()}
        {this.yAxis()}
        {this.legend()}
        {this.bar()}
      </BarChart>
    )
  }
}

export default Chart
